@@ -313,9 +313,9 @@ class ESNRegressor(BaseEstimator, MultiOutputMixin, RegressorMixin):
         if self.requires_sequence == "auto":
             self._check_if_sequence(X, y)
         if self.requires_sequence:
-            self._input_to_node.fit(X[0])
-            self._node_to_node.fit(self._input_to_node.transform(X[0]))
             X, y, sequence_ranges = concatenate_sequences(X, y)
+            self._input_to_node.fit(X)
+            self._node_to_node.fit(self._input_to_node.transform(X))
         else:
             self._validate_data(X, y, multi_output=True)
             self._input_to_node.fit(X)
@@ -512,16 +512,35 @@ class ESNRegressor(BaseEstimator, MultiOutputMixin, RegressorMixin):
         """
         self._node_to_node = node_to_node
 
-    @property
-    def hidden_layer_state(self) -> np.ndarray:
+    def hidden_layer_state(self, X: np.ndarray) -> np.ndarray:
         """
-        Return the hidden_layer_state, e.g. the resevoir state over time.
+        Return the hidden_layer_state, e.g. the reservoir state over time.
+
+        Parameters
+        ----------
+        X : ndarray of shape (n_samples, n_features)
 
         Returns
         -------
-        hidden_layer_state : np.ndarray
+        hidden_layer_state : ndarray of (n_samples,)
+            The hidden_layer_state, e.g. the reservoir state over time.
         """
-        return self._node_to_node._hidden_layer_state
+        if self._input_to_node is None:
+            raise NotFittedError(self)
+
+        if self.requires_sequence is False:
+            # input_to_node
+            hidden_layer_state = self._input_to_node.transform(X)
+            hidden_layer_state = self._node_to_node.transform(
+                hidden_layer_state)
+        else:
+            hidden_layer_state = np.empty(shape=X.shape, dtype=object)
+            for k, seq in enumerate(X):
+                # input_to_node
+                hls = self._input_to_node.transform(seq)
+                hls = self._node_to_node.transform(hls)
+                hidden_layer_state[k] = hls
+        return hidden_layer_state
 
     @property
     def sequence_to_value(self) -> bool:
@@ -718,11 +737,11 @@ class ESNClassifier(ESNRegressor, ClassifierMixin):
         if self.requires_sequence == "auto":
             self._check_if_sequence(X, y)
         if self.requires_sequence:
-            self._input_to_node.fit(X[0])
-            self._node_to_node.fit(self._input_to_node.transform(X[0]))
             self._check_if_sequence_to_value(X, y)
             X, y, sequence_ranges = concatenate_sequences(
                 X, y, sequence_to_value=self._sequence_to_value)
+            self._input_to_node.fit(X)
+            self._node_to_node.fit(self._input_to_node.transform(X))
         else:
             self._validate_data(X, y, multi_output=True)
             self._input_to_node.fit(X)
